/**
 * @author Sandro <sandro.pedrazzini@canoo.com>
 */
package com.canoo.ulc.community.datechooser.client.widget;

import com.canoo.ulc.community.datechooser.client.widget.util.GraphicMemento;
import com.canoo.ulc.community.datechooser.client.widget.util.LocaleAdapter;
import com.canoo.ulc.community.datechooser.client.widget.util.PickDatePanel;

import javax.swing.*;
import javax.swing.plaf.basic.BasicButtonUI;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.text.DateFormat;
import java.util.*;
import java.util.List;

/**
 * GUI component that lets the user select a date.
 * Used as base for the realization of ULCdateChooser.
 * <p/>
 * Changes from the first version
 * 9.6.2004: added clear cell function, considering locales
 * 2.6.2004: added localized adapter functions to correct day of week value when used with internationalization
 * 11.6.2004: button textures as parameters (not internationalized)
 * 12.6.2004 refactored*
 * 16.6.2004: adapted cells'size (in order to be able to write day names using 3 letters): added constants
 * 17.6.2004: localized days headers are now dynamically generated, using the locale
 * 20.6.2004: show current cell when passing with the cursor
 * 24.6.2004: isolated LocaleAdapter, managing also button text.
 * 24.6.2004: substituted todayHiglight color with bigger font
 * 25.6.2004: added GraphicMemento
 * 25.6.2004: day highlight only for the current real day
 * 02.7.2004: DatePanel divided in subpanels, using Box, Border and GridLayout.
 * 06.7.2004: Added DateSelectedListener management
 * 02.08.2004: eliminated TextField in DateChooser
 * <p/>
 * TODO:
 * manage panel position within the screen
 */
public final class DateChooser extends JButton {

    private List fDateSelectedListenersList = new ArrayList();
    private static final Icon sMiniViewIcon = new ImageIcon(DateChooser.class.getResource("/miniview.png"));


    // The current PickDatePanel container
    private JDialog fDialog;

    // Button state
    Date fSelectedDate = null;
    Date fFromDate, fToDate;
    LocaleAdapter fLocaleAdapter;
    GraphicMemento fGraphicMemento;

    public DateChooser() {
        super();
        fLocaleAdapter = new LocaleAdapter();
        fGraphicMemento = new GraphicMemento();
        init();
    }

    public DateChooser(Locale locale, String cancelButtonText) {
        super();
        fLocaleAdapter = new LocaleAdapter(locale, cancelButtonText);
        fGraphicMemento = new GraphicMemento();
        init();
    }

    public DateChooser(Locale locale, String cancelButtonText, Date initialDate) {
        this(locale, cancelButtonText);
        fSelectedDate = initialDate;
    }

    public Date getFromDate() {
        return fFromDate;
    }

    public void setFromDate(Date fromDate) {
        fFromDate = fromDate;
    }

    public Date getToDate() {
        return fToDate;
    }

    public void setToDate(Date toDate) {
        fToDate = toDate;
    }

    /**
     * API get function: returns null if no date has been selected
     */
    public Date getSelectedDate() {
        return fSelectedDate;
    }

    public String getSelectedDateAsString() {
        return dateToString(fSelectedDate, fLocaleAdapter.getLocale(), DateFormat.LONG);
    }

    public void setSelectedDate(Date date) {
        fSelectedDate = date;
    }

    public void enableCurrentDayView(){
        fGraphicMemento.enableCurrentDayView();
    }

    public void disableCurrentDayView(){
        fGraphicMemento.disableCurrentDayView();
    }

    public Dimension getDayCellDimension() {
        return new Dimension(fGraphicMemento.getCellWidth(), fGraphicMemento.getCellHight());
    }

    public void setDayCellDimension(Dimension dimension) {
        fGraphicMemento.setCellWidth((int) dimension.getWidth());
        fGraphicMemento.setCellHight((int) dimension.getHeight());
    }

    public Color getEnabledCellColor() {
        return fGraphicMemento.getEnabledCellColor();
    }

    public void setEnabledCellColor(Color color) {
        fGraphicMemento.setEnabledCellColor(color);
    }

    public Color getDisabledCellColor() {
        return fGraphicMemento.getDisabledCellColor();
    }

    public void setDisabledCellColor(Color color) {
        fGraphicMemento.setDisabledCellColor(color);
    }

    public Color getDialogBackgroundColor() {
        return fGraphicMemento.getDialogBackgroundColor();
    }

    public void setDialogBackgroundColor(Color color) {
        fGraphicMemento.setDialogBackgroundColor(color);
    }

    public void addDateSelectedListener(DateSelectedListener listener) {
        fDateSelectedListenersList.add(listener);
    }

    public void removeDateSelectedListener(DateSelectedListener listener) {
        fDateSelectedListenersList.remove(listener);
    }


    /**
     * Initializes the button components.
     */
    private void init() {
        fSelectedDate = null;
        setIcon(sMiniViewIcon);
        setUI(new BasicButtonUI());
//        setBorderPainted(false);
        setBorder(BorderFactory.createEmptyBorder());
//        setFocusPainted(false);
        
        setOpaque(false);
//        this.setMargin(new Insets(0,0,0,0));
        
        setMargin(fGraphicMemento.getInsets());
        setMinimumSize(new Dimension(fGraphicMemento.getCellWidth(), fGraphicMemento.getCellHight()));
        addActionListener(new CreatePanelAction());
    }

    /**
     * Event handler that displays the PickDatePanel when the button is
     * clicked.
     */
    class CreatePanelAction implements ActionListener {
        public void actionPerformed(final ActionEvent evt) {

            PickDatePanel pickDatePanel = new PickDatePanel(fGraphicMemento, fLocaleAdapter, fSelectedDate, fFromDate, fToDate);
            pickDatePanel.addComponentListener(new DateChoiceListener());

            fDialog = new JDialog(frameForActionEvent(evt), true);
            fDialog.setResizable(false);
            //TODO: (sp) use with 1.4: fDialog.setUndecorated(true);
            fDialog.getContentPane().add(pickDatePanel);
            fDialog.pack();
            
            final Point p = getLocationOnScreen();
            p.setLocation(p.getX() + getSize().getWidth() - fDialog.getWidth(), p.getY() - 1 + getSize().getHeight());
            fDialog.setLocation(p);
            
            fDialog.setVisible(true);
        }
    }
    
    /**
     * Return the <code>Frame</code> owner for the specified <code>ActionEvent</code>.
     * 
     * @param e <code>ActionEvent</code>
     * @return <code>Frame</code> owner
     */
    Frame frameForActionEvent(ActionEvent e) {
        if (e.getSource() instanceof Component) {
            Component c = (Component)e.getSource();
            while(c != null) {
                if (c instanceof Frame) {
    	        return (Frame)c;
                }
                c = (c instanceof JPopupMenu) ? ((JPopupMenu)c).getInvoker() : c.getParent();
            }
        }
        return null;
    }

    /**
     * Returns a short string representation for the specified date (January 1, 2003).
     * Only used for internal test.
     */
    private static String dateToString(final Date date, Locale locale, int dateFormat) {
        if (null != date)
            return DateFormat.getDateInstance(dateFormat, locale).format(date);
        return null;
    }


    /**
     * DateChoiceListener that will catch the selected date when the PickDatePanel is hidden.
     */
    class DateChoiceListener extends ComponentAdapter {
        public void componentHidden(final ComponentEvent evt) {
            final Date selectedDate = ((PickDatePanel) evt.getSource()).getDate();
            //if (PickDatePanel.betweenFromAndTo(selectedDate,fFromDate,fToDate)) {
                if (selectedDate != null) {
                    fSelectedDate = selectedDate;
                    //TODO: (sp) 1.4: evt.setSource(DateChooser.this);
                    Iterator iterator = fDateSelectedListenersList.iterator();
                    while (iterator.hasNext()) {
                        DateSelectedListener listener = (DateSelectedListener) iterator.next();
                        listener.dateChosen(new ComponentEvent(DateChooser.this, 1));
                    }
                }
                fDialog.dispose();
            //}
        }
    }

}
