package com.canoo.ulc.community.filetransfer.application;

import com.canoo.ulc.community.progresspane.application.TaskControllerAdapter;
import com.canoo.ulc.community.progresspane.application.ITaskList;
import com.canoo.ulc.community.progresspane.application.ITask;
import com.canoo.ulc.community.progresspane.application.AbstractSynchronousTask;
import com.canoo.ulc.community.filetransfer.server.ULCFileUploader;

/**
 * Represents a task list that orchestrates the file upload process. The actual file uploading
 * is done by the ULCFileUploader class.
 *
 * @author christian.stettler@canoo.com
 */
public class FileUploadTaskList extends TaskControllerAdapter implements ITaskList {
    private long fBytesPerTask;
    private ULCFileUploader fFileUploader;
    private boolean fUploadComplete;
    private boolean fCancelled;
    private String fClientFilePath;

    public FileUploadTaskList(long bytesPerTask, IFileUploadHandler handler) {
        fBytesPerTask = bytesPerTask;
        fFileUploader = new ULCFileUploader(handler);
        fClientFilePath = fFileUploader.getClientFilePath();

        fUploadComplete = false;
        fCancelled = false;
    }

    public ITask next() {
        return new AbstractSynchronousTask() {
            protected void synchronousExecute() throws Exception {
                if (fCancelled) {
                    fUploadComplete = true;
                    fFileUploader.fireUploadCancelled();
                } else if (fFileUploader.isFailed()) {
                    fUploadComplete = true;
                    fFileUploader.fireUploadFailed();
                } else if (fFileUploader.isCompleted()) {
                    fUploadComplete = true;
                    fFileUploader.fireUploadComplete();
                } else {
                    fFileUploader.uploadNextBytes(fBytesPerTask);
                }
            }

            public String getStatus() {
                if (!fFileUploader.isCompleted()) {
                    return "Downloading " + fClientFilePath + " (" + (fFileUploader.getUploadedByteCount() / 1024) + "kB) ...";
                } else {
                    return "Saving " + fClientFilePath + " ...";
                }
            }
        };
    }

    public boolean hasMore() {
        return !fUploadComplete;
    }

    public double getPercentCompleted() {
        if (fFileUploader.getFileSize() == -1) {
            return fUploadComplete ? 1 : 0;
        } else {
            return (double)fFileUploader.getUploadedByteCount() / (double)fFileUploader.getFileSize();
        }
    }

    public boolean isCancellable() {
        return true;
    }

    public void cancel() {
        fCancelled = true;
    }

    public void error(Exception reason) {
        fFileUploader.fireUploadFailed(reason);
    }

    public void stopped() {
        cancel();
    }
}
