package com.canoo.ulc.community.filetransfer.client;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import com.ulcjava.base.client.ClientEnvironmentAdapter;
import com.ulcjava.base.client.FileContents;
import com.ulcjava.base.client.IFileService;
import com.ulcjava.base.client.UIProxy;
import com.ulcjava.base.shared.internal.Anything;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class UIFileDownloader extends UIProxy {

    private String clientFilePath;

    private IFileService fileService;

    private InputStream fileContentStream;

    private boolean failed;

    public UIFileDownloader() {
        fileContentStream = null;
        failed = false;
    }

    public void setClientFilePath(String clientFilePath) {
        this.clientFilePath = clientFilePath;
    }

    protected void preInitializeState() {
        super.preInitializeState();

        fileService = ClientEnvironmentAdapter.getFileService();

        if (fileService == null) {
            sendDownloadFailed("File service not available");
        }
    }

    public void handleULCRequest(String request, Anything args) {
        if (request.equals("downloadBytes")) {
            if (!failed) {
                downloadNextBytes(args.toLong());
            }
        } else if (request.equals("downloadCancelled")) {
            closeFileContentStream();
        } else if (request.equals("downloadFailed")) {
            failed = true;
            closeFileContentStream();
        }
    }

    private void downloadNextBytes(final long bytesToDownload) {
        try {
            if (fileContentStream == null) {
                FileContents fileContents = fileService.openFile(clientFilePath);
                fileContentStream = fileContents.getInputStream();

                long downloadSize = new File(fileContents.getFilePath()).length();
                // sendULC("downloadSize", new Anything(downloadSize <= 0 ? -1 :
                // downloadSize));
                invokeULC("handleUIRequest", new Object[] { "downloadSize",
                        new Anything(downloadSize <= 0 ? -1 : downloadSize) });
            } else {
                ByteReadingRunnable byteReadingRunnable = new ByteReadingRunnable(bytesToDownload);
                byteReadingRunnable.run();

                if (byteReadingRunnable.getFailedReason() != null) {
                    sendDownloadFailed(byteReadingRunnable.getFailedReason().getMessage());
                } else {
                    invokeULC("handleUIRequest", new Object[] { "downloadBytes",
                            new Anything(byteReadingRunnable.getDownloadedBytes()) });
                }
            }
        } catch (Exception e) {
            sendDownloadFailed(e.getMessage());
        }
    }

    private void sendDownloadComplete() {
        closeFileContentStream();
        invokeULC("handleUIRequest", new Object[] { "downloadComplete", null });
        sendMarkCollectableULC();
    }

    private void sendDownloadFailed(String reason) {
        closeFileContentStream();
        invokeULC("handleUIRequest", new Object[] { "downloadFailed", new Anything(reason) });
        sendMarkCollectableULC();

        failed = true;
    }

    private void closeFileContentStream() {
        if (fileContentStream != null) {
            try {
                fileContentStream.close();
            } catch (IOException ignored) {
            }
        }
    }

    private class ByteReadingRunnable implements Runnable {

        private long bytesToDownload;

        private byte[] downloadedBytes;

        private Exception failedReason;

        public ByteReadingRunnable(long bytesToDownload) {
            this.bytesToDownload = bytesToDownload;
            this.downloadedBytes = new byte[0];
            this.failedReason = null;
        }

        public void run() {
            ByteArrayOutputStream out = null;

            try {
                byte[] buffer = new byte[(int) Math.min(1024, bytesToDownload)];
                int byteCountForCurrentRead;
                if ((byteCountForCurrentRead = fileContentStream.read(buffer)) == -1) {
                    sendDownloadComplete();
                } else {
                    out = new ByteArrayOutputStream();

                    int bytesRead = 0;

                    out.write(buffer, 0, byteCountForCurrentRead);
                    bytesRead += byteCountForCurrentRead;

                    while (bytesRead < bytesToDownload
                            && (byteCountForCurrentRead = fileContentStream.read(buffer)) != -1) {
                        out.write(buffer, 0, byteCountForCurrentRead);
                        bytesRead += byteCountForCurrentRead;
                    }

                    downloadedBytes = out.toByteArray();
                }
            } catch (Exception e) {
                failedReason = e;
            } finally {
                if (out != null) {
                    try {
                        out.close();
                    } catch (IOException ignored) {
                    }
                }
            }
        }

        public byte[] getDownloadedBytes() {
            return downloadedBytes;
        }

        public Exception getFailedReason() {
            return failedReason;
        }
    }
}
