package com.canoo.ulc.community.filetransfer.application;

import com.ulcjava.base.application.*;
import com.ulcjava.base.application.datatype.ULCNumberDataType;
import com.ulcjava.base.application.enabler.ULCAndEnabler;
import com.ulcjava.base.application.event.ActionEvent;
import com.ulcjava.base.application.event.IActionListener;
import com.ulcjava.base.application.util.HTMLUtilities;
import com.ulcjava.base.application.util.IFileChooseHandler;
import com.ulcjava.base.shared.FileChooserConfig;

import java.text.DecimalFormatSymbols;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class FileDownloadPane extends ULCBoxPane {
    private static final int UPLOAD_CHUNK_SIZE_IN_KB = 10;
    private static final long MAXIMUM_FILE_SIZE_IN_BYTES = 5 * 1024 * 1024;

    private ULCLabel fFileUploadDescriptionLabel;
    private ULCButton fFileUploadButton;
    private ULCLabel fUploadChunkSizeLabel;
    private ULCTextField fUploadChunkSizeTextField;

    public FileDownloadPane() {
        super(4, 0, 10, 10);

        initComponents();
        addListenersToComponents();
        layoutComponents();
    }

    public String getName() {
        return "File Transfer: Client to Server";
    }

    private void initComponents() {
        ULCNumberDataType numberDataType = new ULCNumberDataType("#0", new DecimalFormatSymbols());
        numberDataType.setInteger(true);
        numberDataType.setMin(0);
        numberDataType.setMax(1024);

        fFileUploadDescriptionLabel = new ULCLabel(HTMLUtilities.convertToHtml("" +
                "<b>Download a file from the client to the server</b>\n\n" +
                "The file gets transferred from the client to the server in multiple chunks, " +
                "each of the size specified by the application developer. There is a trade-off between " +
                "the number of server round-trips and the quality of the visual progress representation: " +
                "the better the progress visualization, the larger the number of server round-trips and thus, " +
                "the larger the communication overhead (bytes sent, latency). " +
                "After successful completion, the content of file sent from the client to the server " +
                "is made available to the server as a byte array for further processing.\n\n" +
                "From the client's point of view, this action is a file upload." +
                ""));

        fUploadChunkSizeLabel = new ULCLabel("Chunk Size [kB]");
        fUploadChunkSizeTextField = new ULCTextField(String.valueOf(UPLOAD_CHUNK_SIZE_IN_KB), 4);
        fUploadChunkSizeTextField.setDataType(numberDataType);

        fFileUploadButton = new ULCButton("Start");
    }

    private void addListenersToComponents() {
        fFileUploadButton.addActionListener(new FileDownloadAction());

        ULCAndEnabler fileUploadButtonEnabler = new ULCAndEnabler();
        fileUploadButtonEnabler.add(fUploadChunkSizeTextField);
        fFileUploadButton.setEnabler(fileUploadButtonEnabler);
    }

    private void layoutComponents() {
        setBorder(Utils.createMarginBorder(10));

        add(getColumns(), ULCBoxPane.BOX_EXPAND_TOP, fFileUploadDescriptionLabel);
        add(getColumns(), ULCBoxPane.BOX_EXPAND_EXPAND, new ULCFiller());

        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadChunkSizeLabel);
        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadChunkSizeTextField);
        add(ULCBoxPane.BOX_EXPAND_CENTER, new ULCFiller(10, 0));
        add(ULCBoxPane.BOX_RIGHT_CENTER, fFileUploadButton);
    }

    private class FileDownloadAction implements IActionListener {
        public void actionPerformed(ActionEvent event) {
            FileChooserConfig fileChooserConfig = new FileChooserConfig();
            fileChooserConfig.setDialogType(FileChooserConfig.OPEN_DIALOG);

            ClientContext.chooseFile(new IFileChooseHandler() {
                public void onSuccess(final String filePath) {
                    FileDownloadTaskList fileUploadTaskList = new FileDownloadTaskList(MAXIMUM_FILE_SIZE_IN_BYTES, getIntValue(fUploadChunkSizeTextField) * 1024, new IFileDownloadHandler() {
                        public String getClientFilePath() {
                            return filePath;
                        }

                        public void onSuccess(byte[] downloadedBytes) {
                            Utils.showMessageDialog("Info", "File upload completed sucessfully.\nFile size is " + downloadedBytes.length / 1024 + "kB", ULCAlert.INFORMATION_MESSAGE);
                        }

                        public void onFailure(Exception reason) {
                            Utils.showMessageDialog("Error", "File upload failed:\n" + reason.getMessage(), ULCAlert.ERROR_MESSAGE);
                        }

                        public void onCancel() {
                            Utils.showMessageDialog("Info", "File upload cancelled", ULCAlert.INFORMATION_MESSAGE);
                        }
                    });

                    Utils.startWithProgressDialog(fileUploadTaskList, "File Upload");
                }

                public void onFailure(int reason, String description) {
                    if (reason != IFileChooseHandler.CANCELLED) {
                        Utils.showMessageDialog("Error", "File choosing failed:\n" + description, ULCAlert.ERROR_MESSAGE);
                    }
                }
            }, fileChooserConfig, null);
        }

        private int getIntValue(ULCTextField integerTextField) {
            return ((Integer) integerTextField.getValue()).intValue();
        }
    }
}
