package com.canoo.ulc.community.filetransfer.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import com.canoo.ulc.community.filetransfer.application.IFileDownloadHandler;
import com.ulcjava.base.application.ULCProxy;
import com.ulcjava.base.shared.internal.Anything;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class ULCFileDownloader extends ULCProxy {

    private IFileDownloadHandler fHandler;

    private boolean fCompleted;

    private ByteArrayOutputStream fDownloadedByteStream;

    private long fDownloadedByteCount;

    private long fDownloadSize;

    private Exception fFailedReason;

    private boolean fFailedOnClient;

    private long fMaximumDownloadSize;

    public ULCFileDownloader(IFileDownloadHandler handler, long maximumFileSizeInBytes) {
        fHandler = handler;

        fCompleted = false;
        fFailedOnClient = false;
        fFailedReason = null;
        fDownloadedByteStream = new ByteArrayOutputStream();
        fDownloadedByteCount = 0;
        fDownloadSize = -1;
        fMaximumDownloadSize = maximumFileSizeInBytes;

        markUncollectable();
        upload();
    }

    public void downloadNextBytes(long byteCount) {
        if (fCompleted) {
            throw new IllegalStateException("Download already completed");
        }
        if (fFailedReason != null) {
            throw new IllegalStateException("Download failed: " + fFailedReason.getMessage());
        }

        invokeUI("handleULCRequest", new Object[] { "downloadBytes", new Anything(byteCount) });
    }

    public void fireDownloadComplete() {
        byte[] downloadedBytes = fDownloadedByteStream.toByteArray();

        closeDownloadedByteStream();
        fHandler.onSuccess(downloadedBytes);
    }

    public void fireDownloadCancelled() {
        closeDownloadedByteStream();
        fHandler.onCancel();

        invokeUI("handleULCRequest", new Object[] { "downloadCancelled", null });
    }

    public void fireDownloadFailed() {
        fireDownloadFailed(fFailedReason);
    }

    public void fireDownloadFailed(Exception reason) {
        closeDownloadedByteStream();
        fHandler.onFailure(reason);

        if (!fFailedOnClient) {
            invokeUI("handleULCRequest", new Object[] { "downloadFailed", null });
        }
    }

    public String getClientFilePath() {
        return fHandler.getClientFilePath();
    }

    public long getDownloadSize() {
        return fDownloadSize;
    }

    public long getDownloadedByteCount() {
        return fDownloadedByteCount;
    }

    public boolean isCompleted() {
        return fCompleted;
    }

    public boolean isFailed() {
        return fFailedReason != null;
    }

    protected void uploadStateUI() {
        super.uploadStateUI();

        setStateUI("clientFilePath", null, fHandler.getClientFilePath());
    }

    public void handleUIRequest(String request, Anything args) {
        if (request.equals("downloadBytes")) {
            try {
                byte[] downloadedBytes = (byte[]) args.toObject();
                fDownloadedByteCount += downloadedBytes.length;
                fDownloadedByteStream.write(downloadedBytes);
            } catch (IOException e) {
                fFailedReason = e;
            }
        } else if (request.equals("downloadSize")) {
            fDownloadSize = args.toLong();

            checkDownloadSize(fDownloadSize);
        } else if (request.equals("downloadComplete")) {
            fCompleted = true;
        } else if (request.equals("downloadFailed")) {
            fFailedOnClient = true;
            fFailedReason = new Exception(args.toString());
        }
    }

    private void checkDownloadSize(long downloadSize) {
        if (fMaximumDownloadSize != -1 && downloadSize > fMaximumDownloadSize) {
            fFailedReason = new Exception("Maximum file size for uploading reached.\nLimit is " + fMaximumDownloadSize
                    / (1024 * 1024) + "MB.");
        }
    }

    private void closeDownloadedByteStream() {
        if (fDownloadedByteStream != null) {
            try {
                fDownloadedByteStream.close();
            } catch (IOException ignored) {
            }

            fDownloadedByteStream = null;
        }
    }

    protected String typeString() {
        return "com.canoo.ulc.community.filetransfer.client.UIFileDownloader";
    }
}
