package com.canoo.ulc.community.filetransfer.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.canoo.ulc.community.filetransfer.application.IFileUploadHandler;
import com.ulcjava.base.application.ULCProxy;
import com.ulcjava.base.shared.internal.Anything;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class ULCFileUploader extends ULCProxy {

    private IFileUploadHandler fHandler;

    private InputStream fUploadedByteStream;

    private boolean fCompleted;

    private boolean fFailed;

    private int fUploadedByteCount;

    private boolean fFailedOnClient;

    private Exception fFailedReason;

    private long fFileSize;

    private boolean fFileSizeRetrieved;

    public ULCFileUploader(IFileUploadHandler handler) {
        fHandler = handler;

        fFileSize = -1;
        fFileSizeRetrieved = false;
        fUploadedByteCount = 0;

        fCompleted = false;
        fFailed = false;
        fFailedOnClient = false;
        fFailedReason = null;
    }

    public String getClientFilePath() {
        return fHandler.getClientFilePath();
    }

    protected void uploadStateUI() {
        super.uploadStateUI();
        setStateUI("clientFilePath", null, fHandler.getClientFilePath());
    }

    public void handleUIRequest(String request, Anything args) {
        if (request.equals("uploadComplete")) {
            fCompleted = true;
        } else if (request.equals("uploadFailed")) {
            fFailedOnClient = true;
            fFailed = true;
            fFailedReason = new Exception(args.toString());
        }
    }

    public void fireUploadComplete() {
        closeUploadedByteStream();
        fHandler.onSuccess();
    }

    public void fireUploadCancelled() {
        closeUploadedByteStream();
        fHandler.onCancel();

        invokeUI("handleULCRequest", new Object[] { "uploadCancelled", null });
    }

    public void fireUploadFailed() {
        fireUploadFailed(fFailedReason);
    }

    public void fireUploadFailed(Exception reason) {
        closeUploadedByteStream();
        fHandler.onFailure(reason);

        if (!fFailedOnClient) {
            invokeUI("handleULCRequest", new Object[] { "uploadFailed", null });
        }
    }

    public boolean isCompleted() {
        return fCompleted;
    }

    public boolean isFailed() {
        return fFailed;
    }

    public long getFileSize() {
        if (!fFileSizeRetrieved) {
            fFileSize = fHandler.getFileSize();
            fFileSizeRetrieved = true;
        }
        return fFileSize;
    }

    public int getUploadedByteCount() {
        return fUploadedByteCount;
    }

    public void uploadNextBytes(long bytesPerTask) {
        if (!isUploaded()) {
            markUncollectable();
            upload();
        }

        if (isCompleted()) {
            throw new IllegalStateException("Upload already completed");
        }
        if (isFailed()) {
            throw new IllegalStateException("Upload failed");
        }

        try {
            if (fUploadedByteStream == null) {
                fUploadedByteStream = fHandler.getInputStream();
            }

            byte[] buffer = new byte[(int) Math.min(1024, bytesPerTask)];
            int byteCountForCurrentRead;
            if ((byteCountForCurrentRead = fUploadedByteStream.read(buffer)) == -1) {
                closeUploadedByteStream();
                invokeUI("handleULCRequest", new Object[] { "storeFile", null });
            } else {
                ByteArrayOutputStream out = null;

                try {
                    out = new ByteArrayOutputStream();

                    int bytesRead = 0;

                    out.write(buffer, 0, byteCountForCurrentRead);
                    bytesRead += byteCountForCurrentRead;

                    while (bytesRead < bytesPerTask && (byteCountForCurrentRead = fUploadedByteStream.read(buffer)) != -1) {
                        out.write(buffer, 0, byteCountForCurrentRead);
                        bytesRead += byteCountForCurrentRead;
                    }

                    fUploadedByteCount += bytesRead;
                    invokeUI("handleULCRequest", new Object[] { "uploadBytes", new Anything(out.toByteArray()) });
                }
                finally {
                    if (out != null) {
                        try {
                            out.close();
                        }
                        catch (IOException ignored) {
                        }
                    }
                }
            }
        }
        catch (Exception e) {
            fFailedReason = e;
            fFailed = true;
        }
    }

    private void closeUploadedByteStream() {
        if (fUploadedByteStream != null) {
            try {
                fUploadedByteStream.close();
            }
            catch (IOException ignored) {
            }
        }
    }

    protected String typeString() {
        return "com.canoo.ulc.community.filetransfer.client.UIFileUploader";
    }
}
