package com.canoo.ulc.community.filetransfer.application;

import com.ulcjava.base.application.*;
import com.ulcjava.base.application.datatype.ULCNumberDataType;
import com.ulcjava.base.application.enabler.ULCAndEnabler;
import com.ulcjava.base.application.event.IActionListener;
import com.ulcjava.base.application.event.ActionEvent;
import com.ulcjava.base.application.util.HTMLUtilities;
import com.ulcjava.base.application.util.IFileChooseHandler;
import com.ulcjava.base.shared.FileChooserConfig;

import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.text.DecimalFormatSymbols;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class FileUploadPane extends ULCBoxPane {
    private static final int UPLOAD_FILE_SIZE_IN_KB = 100;
    private static final int UPLOAD_CHUNK_SIZE_IN_KB = 10;

    private ULCLabel fFileUploadDescriptionLabel;
    private ULCButton fFileUploadButton;
    private ULCLabel fUploadChunkSizeLabel;
    private ULCTextField fUploadChunkSizeTextField;
    private ULCLabel fUploadFileSizeLabel;
    private ULCTextField fUploadFileSizeTextField;

    public FileUploadPane() {
        super(7, 0, 10, 10);

        initComponents();
        addListenersToComponents();
        layoutComponents();
    }

    public String getName() {
        return "File Transfer: Server to Client";
    }

    private void initComponents() {
        ULCNumberDataType numberDataType = new ULCNumberDataType("#0", new DecimalFormatSymbols());
        numberDataType.setInteger(true);
        numberDataType.setMin(0);
        numberDataType.setMax(1024);

        fFileUploadDescriptionLabel = new ULCLabel(HTMLUtilities.convertToHtml("" +
                "<b>Upload a file from the server to the client</b>\n\n" +
                "The file gets transferred from the server to the client in multiple chunks, " +
                "each of the size specified by the application developer. There is a trade-off between " +
                "the number of server round-trips and the quality of the visual progress representation: " +
                "the better the progress visualization, the larger the number of server round-trips and thus, " +
                "the larger the communication overhead (bytes sent, latency). " +
                "After successful completion, the file sent from the server to the client " +
                "is stored at the path chosen by the user.\n\n" +
                "From the client's point of view, this action is a file download." +
                ""));

        fUploadChunkSizeLabel = new ULCLabel("Chunk Size [kB]");
        fUploadChunkSizeTextField = new ULCTextField(String.valueOf(UPLOAD_CHUNK_SIZE_IN_KB), 4);
        fUploadChunkSizeTextField.setDataType(numberDataType);

        fUploadFileSizeLabel = new ULCLabel("File Size [kB]");
        fUploadFileSizeTextField = new ULCTextField(String.valueOf(UPLOAD_FILE_SIZE_IN_KB), 4);
        fUploadFileSizeTextField.setDataType(numberDataType);

        fFileUploadButton = new ULCButton("Start");
    }

    private void addListenersToComponents() {
        fFileUploadButton.addActionListener(new FileUploadAction());

        ULCAndEnabler fileUploadButtonEnabler = new ULCAndEnabler();
        fileUploadButtonEnabler.add(fUploadChunkSizeTextField);
        fileUploadButtonEnabler.add(fUploadFileSizeTextField);
        fFileUploadButton.setEnabler(fileUploadButtonEnabler);
    }

    private void layoutComponents() {
        setBorder(Utils.createMarginBorder(10));

        add(getColumns(), ULCBoxPane.BOX_EXPAND_TOP, fFileUploadDescriptionLabel);
        add(getColumns(), ULCBoxPane.BOX_EXPAND_EXPAND, new ULCFiller());

        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadChunkSizeLabel);
        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadChunkSizeTextField);
        add(ULCBoxPane.BOX_EXPAND_CENTER, new ULCFiller(10, 0));
        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadFileSizeLabel);
        add(ULCBoxPane.BOX_LEFT_CENTER, fUploadFileSizeTextField);
        add(ULCBoxPane.BOX_EXPAND_CENTER, new ULCFiller(10, 0));
        add(ULCBoxPane.BOX_RIGHT_CENTER, fFileUploadButton);
    }

    private class FileUploadAction implements IActionListener {
        public void actionPerformed(ActionEvent event) {
            FileChooserConfig fileChooserConfig = new FileChooserConfig();
            fileChooserConfig.setDialogType(FileChooserConfig.SAVE_DIALOG);

            ClientContext.chooseFile(new IFileChooseHandler() {
                public void onSuccess(final String filePath) {
                    FileUploadTaskList fileUploadTaskList = new FileUploadTaskList(getIntValue(fUploadChunkSizeTextField) * 1024, new IFileUploadHandler() {
                        public InputStream getInputStream() {
                            return new ByteArrayInputStream(createUploadData(getIntValue(fUploadFileSizeTextField)));
                        }

                        public long getFileSize() {
                            return getIntValue(fUploadFileSizeTextField) * 1024;
                        }

                        public String getClientFilePath() {
                            return filePath;
                        }

                        public void onSuccess() {
                            Utils.showMessageDialog("Info", "File download completed sucessfully.\nFile path is " + getClientFilePath(), ULCAlert.INFORMATION_MESSAGE);
                        }

                        public void onFailure(Exception reason) {
                            Utils.showMessageDialog("Error", "File download failed:\n" + reason.getMessage(), ULCAlert.ERROR_MESSAGE);
                        }

                        public void onCancel() {
                            Utils.showMessageDialog("Info", "File download cancelled", ULCAlert.INFORMATION_MESSAGE);
                        }
                    });

                    Utils.startWithProgressDialog(fileUploadTaskList, "File Download");
                }

                public void onFailure(int reason, String description) {
                    if (reason != IFileChooseHandler.CANCELLED) {
                        Utils.showMessageDialog("Error", "File choosing failed:\n" + description, ULCAlert.ERROR_MESSAGE);
                    }
                }
            }, fileChooserConfig, null);
        }

        private int getIntValue(ULCTextField integerTextField) {
            return ((Integer) integerTextField.getValue()).intValue();
        }

        private byte[] createUploadData(int uploadFileSizeInKb) {
            byte[] uploadDate = new byte[uploadFileSizeInKb * 1024];

            for (int i = 0; i < uploadDate.length; i++) {
                if (i > 0 && i % 80 == 0) {
                    uploadDate[i] = '\n';
                } else {
                    uploadDate[i] = (byte) ((i % 50) + 'A');
                }
            }

            return uploadDate;
        }
    }
}
